﻿Type=Class
Version=3.2
@EndOfDesignText@
'Class Name: ModularMenu
#Event: Click(Position As Int, Value As String)

'Author: Jem Miller
'Version: 1.1
'B4A Version Used: 3.20
'Last Modified: 01/11/2014
'-------------------------
'Class module
Private Sub Class_Globals

	Private vActivity As Activity
	Private vModule As Object
	Private pnlMain,pnlMenu As Panel
	Private MenuList As ListView
	
	Private vMaxItemShow, vHeightItem, vPosition As Int
	Private vValue As String
	Private  aWidth As Int
	Private ColorBorder,ColorBackground,ColorText As Int
	Private gi, mlength As Float
	Private ref As Canvas
	Private Dist = 10dip, Corner = 3dip, EdgeIcon = 3dip, TxtSize = 16 As Float
	
	Private ListItem As List
	Private vEventName As String
	
End Sub

'Initializes the ModularMenu.
'MaxItemShow - maximum number of item visible in the menu.
'HeightItem - height of each item.  in DIP units.
'Parent - The Main Activity (Should always be Activity)
'Module - must be Me (with no " marks)
'bgColor - Menu color
'txtColor - Text color
'TextSize - Text size
'Event - Event name to raise on Click event (create an event with this prototype: Event_Click(Position as Int, Value as String).
Public Sub Initialize (MaxItemShow As Int,ItemHeight As Int,Parent As Activity,Module As Object, bgColor As Int, txtColor As Int, TextSize As Float, Event As String)
	'--OPTION SETUP-----------
	
	ColorBorder = Colors.Gray
	ColorBackground = bgColor
	ColorText = txtColor
	TxtSize = TextSize
	
	vEventName = Event&"_click"
	vHeightItem = ItemHeight
	vMaxItemShow = MaxItemShow
	vActivity = Parent
	vModule = Module
	
	aWidth = vActivity.Width
	mlength = 20dip
	ListItem.Initialize
	
	pnlMain.Initialize("pnlMain")
	pnlMain.Color = Colors.Transparent
	pnlMain.Visible = False
	vActivity.AddView(pnlMain,0,0,100%x,100%y)
		
	MenuList.Initialize("MenuList")
	MenuList.Color = ColorBackground
	MenuList.Background = SetRadialCorner(ColorBackground,Corner)
	MenuList.ScrollingBackgroundColor = ColorBackground
	MenuList.FastScrollEnabled = False
	
	MenuList.SingleLineLayout.ItemHeight = vHeightItem
   MenuList.SingleLineLayout.Label.TextSize = TxtSize
   MenuList.SingleLineLayout.Label.TextColor = ColorText
   MenuList.SingleLineLayout.Label.Gravity = Gravity.CENTER_VERTICAL
   
	MenuList.TwoLinesAndBitmap.ItemHeight = MenuList.SingleLineLayout.ItemHeight
	MenuList.TwoLinesAndBitmap.Label.TextSize = MenuList.SingleLineLayout.Label.TextSize
	MenuList.TwoLinesAndBitmap.Label.TextColor = ColorText
	MenuList.TwoLinesAndBitmap.Label.Gravity = Gravity.CENTER_VERTICAL
	MenuList.TwoLinesAndBitmap.Label.Height = vHeightItem
	
	MenuList.TwoLinesAndBitmap.Label.Left = vHeightItem + 6dip + EdgeIcon * 2
	MenuList.TwoLinesAndBitmap.Label.Top = 0
	MenuList.TwoLinesAndBitmap.SecondLabel.Visible = False
	MenuList.TwoLinesAndBitmap.ImageView.Gravity = Gravity.FILL
	MenuList.TwoLinesAndBitmap.ImageView.SetLayout(6dip + EdgeIcon, EdgeIcon, vHeightItem - EdgeIcon * 2, vHeightItem - EdgeIcon * 2)
	
	RefillMenu
	
	pnlMenu.Initialize("")
	pnlMenu.Color = ColorBorder
	
	pnlMenu.Background = SetRadialCorner(ColorBorder,Corner)
	pnlMain.AddView(pnlMenu,0,0,20,40)
	pnlMenu.AddView(MenuList,0,0,0,0)
	
	ref.Initialize(pnlMenu)
	
End Sub

Private Sub ShowMenu

	If ListItem.Size = 0 Then
		ToastMessageShow("No menu items in the list.",False)
		Return
	End If
	
	ResizeMenu
	
End Sub

Private Sub ShowMenuAt(L As Int, T As Int)

	If ListItem.Size = 0 Then
		ToastMessageShow("No menu items in the list.",False)
		Return
	End If
	
	ResizeMenuAt(L,T)
	
End Sub

Private Sub ResizeMenu
	Dim PosX,PosY,Height = HeightMenu As Int
		
	If vActivity.Width > vActivity.Height Then 'Landscape
		PosX = 100%x - mlength - Dist
	Else 'Portrait
		PosX = 50%x-mlength / 2
	End If
	
	PosY = 100%y - Height + 5dip
	pnlMenu.SetLayout(PosX,PosY,mlength,Height)
	
	MenuList.SetLayout(1dip,1dip, mlength - 2dip, Height - 2dip)
	pnlMain.BringToFront
	pnlMain.Visible = True
	
	MenuList.BringToFront
	
End Sub

Private Sub ResizeMenuAt(l,t As Int)
	Dim i,PosX,Height = HeightMenu As Int
		
	i = (l + mlength + Dist)
	
	If i > aWidth Then
	PosX = aWidth - (i - l)
	Else
	PosX = l
	End If
	
	If PosX < 0 Then
	PosX = 0
	End If
	
	pnlMenu.SetLayout(PosX, t, mlength, Height)
	MenuList.SetLayout(1dip,1dip, mlength - 2dip, Height - 2dip)
	pnlMain.Visible = True
	
	MenuList.BringToFront
	
End Sub

'Programmatically closes the menu.
Public Sub CloseMenu
	
	DoEvents
	
	pnlMain.Visible = False
	
End Sub

'Adds an item to the menu.
'Title - title displayed in the menu.
'Image - Pass Null if there is no image to load.
Public Sub AddMenuItem(Title As String,Image As Bitmap)
	
	gi = ref.MeasureStringWidth(Title, Typeface.DEFAULT, TxtSize) + 20dip
	
	If Image.IsInitialized Then
	gi = gi + vHeightItem + 20dip
	End If
	
	If gi > mlength Then
	mlength = gi
	End If
	
	ListItem.Add(Title)
	
	If Not(Image.IsInitialized) Then 
	ListItem.Add(Null) 
	Else 
	ListItem.Add(Image)
	End If
	
	RefillMenu
	
End Sub

'Adds an item to the menu in the specified position.
'Title - title displayed in the menu.
'Image - put Null if there is no image to load.
'Position - position in the menu.
Public Sub AddMenuItemAt(Title As String,Image As Bitmap,Position As Int)
	
	gi = ref.MeasureStringWidth(Title, Typeface.DEFAULT, TxtSize) + 20dip
	
	If Image.IsInitialized Then
	gi = gi + vHeightItem + 20dip
	End If
	
	If gi > mlength Then
	mlength = gi
	End If
	
	Position = Position * 2
	
	If Not(Image.IsInitialized) Then 
	ListItem.InsertAt(Position,Null) 
	Else 
	ListItem.InsertAt(Position,Image)
	End If
	
	ListItem.InsertAt(Position,Title)
	
	RefillMenu
	
End Sub

'Removes the item at the specified position.
Public Sub RemoveAt(Index As Int) As Boolean
	Dim c As Int
	
	If MenuList.IsInitialized = False Then Return False
	
	If Index < 0 OR Index > MenuList.Size-1 Then Return False
	
	MenuList.RemoveAt(Index)
	
	For c = 0 To 1
		ListItem.RemoveAt(0 + Index * 2)
	Next
	
	mlength = 0
	
	For c = 0 To MenuList.Size - 1
	gi = ref.MeasureStringWidth(MenuList.GetItem(c), Typeface.DEFAULT, TxtSize)
	
	If gi > mlength Then
	mlength = gi
	End If
	
	Next
	
	pnlMenu.Width = mlength
	MenuList.Width = mlength - 2dip
	DoEvents
	
	Return True
	
End Sub

'Removes all items from the menu.
Public Sub RemoveAll As Boolean

	If MenuList.IsInitialized = False Then Return False
	
	MenuList.Clear
	
	mlength = 40dip
	
	pnlMenu.Width = mlength
	MenuList.Width = mlength - 2dip
	DoEvents
	
	Return True
	
End Sub

'Returns the number of menu items  [read only].
Public Sub Size As Int

	If ListItem.Size = 0 Then Return 0
	
	Return ListItem.Size / 2 +1
	
End Sub

'Programmatically opens the menu.
Public Sub OpenMenu

	ShowMenu
	
End Sub

'Programmatically opens the menu at a specified location.
'Left - Left edge of menu.
'Top - Top edge of menu.
Public Sub OpenMenuAt(Left As Int, Top As Int)

	ShowMenuAt(Left, Top)
	
End Sub

'Returns True if menu is currently showing.
Public Sub IsVisible As Boolean
	
	If pnlMain.IsInitialized Then
	Return pnlMain.Visible
	Else
	Return False
	End If
	
End Sub

Private Sub MenuList_ItemClick (Position As Int, Value As Object)

	vPosition = Position
	vValue = Value
	
	If vEventName <>"" Then
	
		If SubExists(vModule,vEventName) Then
			CallSub3(vModule,vEventName,vPosition,vValue)
		End If
		
	End If
	
	CloseMenu
	
End Sub

Private Sub RefillMenu
	Dim c As Int
	Dim Title As String
	
	MenuList.Clear
	
	If ListItem.Size > 0 Then
		For c = 0 To ListItem.Size / 2 - 1
			Title = ListItem.Get(0 + c * 2)
			
			Dim Image As Bitmap
			
			If ListItem.Get(1 + c * 2) <> Null Then
				Image.Initialize3(ListItem.Get(1 + c * 2))
			End If
			
			If Image.IsInitialized = False Then
				MenuList.AddSingleLine(Title)
			Else
				MenuList.AddTwoLinesAndBitmap(Title,"", Image)
			End If
			
		Next
		
	End If
	
End Sub

Private Sub HeightMenu As Int
	Dim Height,Diff = 1.5 As Float
	Dim Qty As Int
	
	Height = MenuList.SingleLineLayout.ItemHeight+Diff
	Qty = Min(ListItem.Size / 2, vMaxItemShow)
	
	If Qty > (100%y - Dist / 2)/Height Then Qty = (100%y - Dist / 2)/Height
	
	Return Height * Qty + 5 - Diff
	
End Sub

Private Sub SetRadialCorner (Color As Int,Radius As Int) As GradientDrawable
	Dim grdDwb As GradientDrawable
	Dim Colore(2) As Int
	
	Colore(0) = Color
	Colore(1) = Colore(0)
	grdDwb.Initialize("TOP_BOTTOM",Colore)
	grdDwb.CornerRadius = Radius
	Return grdDwb
	
End Sub

Private Sub pnlMain_Click
	
	pnlMain.Visible = False
	
End Sub
